
const addCatModal = document.getElementById("add-cat-modal");
const addCatButton = document.getElementById("add-cat-button");

function openAddCatModal() {
    addCatModal.style.display = "block";
}

function closeAddCatModal() {
    addCatModal.style.display = "none";
}

window.onclick = function (event) {
    if (event.target === addCatModal) {
        closeAddCatModal();
    }
}
function redirectToCatPage(catId) {
    // Construct the URL with the cat_id
    const catUrl = `/cat/${catId}`;

    // Redirect to the cat page
    window.location.href = catUrl;
}


function editCatForm(event) {
    event.preventDefault();

    const catId = document.getElementById("edit-cat-id").value;

    const formData = new FormData();
    formData.append("name", document.getElementById("edit-name").value);
    formData.append("description", document.getElementById("edit-description").value);
    formData.append("location", document.getElementById("edit-location").value);
    formData.append("gender", document.getElementById("edit-gender").value);
    formData.append("color", document.getElementById("edit-color").value);
    formData.append("spayed", document.getElementById("edit-spayed").value);
    formData.append("image_file", document.getElementById("edit-image_file").files[0]);
    formData.append("sound_file", document.getElementById("edit-sound_file").files[0]);

    // Send a POST request to /cats/{cat_id}
    fetch(`/cat/${catId}`, {
        method: "POST",
        body: formData,
    })
        .then((response) => {
            if (response.ok) {

                closeEditCatModal();
                location.reload();

            } else {
                console.error("Error editing cat:", response.statusText);
            }
        })
        .catch((error) => {
            console.error("Error editing cat:", error);
        });
}


function addCatSubmit(event) {
    event.preventDefault();
    const addCatForm = document.getElementById("add-cat-form");

    const formData = new FormData(addCatForm);

    fetch("/cat", {
        method: "POST",
        body: formData,
    })
        .then((response) => {
            if (response.ok) {

                console.log("Cat added successfully");
                closeAddCatModal();
                window.location.reload();
            } else {
                console.error("Failed to add cat");
            }
        })
        .catch((error) => {
            console.error("Error:", error);
        });
}

function getQueryParam(name) {
    const urlParams = new URLSearchParams(window.location.search);
    return urlParams.get(name);
}

function setFormFieldValue(searchForm, fieldName, value) {

    const field = searchForm.querySelector(`[name='${fieldName}']`);
    if (field) {
        if (field.type === 'checkbox') {
            // If it's a checkbox, set the 'checked' property
            field.checked = value === 'true';
        } else {
            field.value = value;
        }
    }
}



// JavaScript to handle delete button click
document.addEventListener("DOMContentLoaded", function () {
    const deleteButtons = document.querySelectorAll(".delete-button");
    const editButton = document.querySelectorAll(".edit-button");

    editButton.forEach(function (button) {
        button.addEventListener("click", function (event) {
            event.stopPropagation();
        })
    });

    deleteButtons.forEach(function (button) {
        button.addEventListener("click", function (event) {
            event.stopPropagation();
            event.preventDefault();
            let conf = confirm('Are you sure you want to delete this cat? This will delete all sightings for the cat too.');
            if (!conf) {
                event.preventDefault();
                return;
            }


            const deleteUrl = button.getAttribute("href");


            fetch(deleteUrl, {
                method: "DELETE",
            })
                .then(function (response) {
                    if (response.ok) {
                        window.location.href = "/cat";
                        location.reload();
                    } else {

                        console.error("DELETE request failed.");
                    }
                })
                .catch(function (error) {

                    console.error("Network error:", error);
                });
        });
    });

    const searchForm = document.getElementById("search-form");
    let debounceTimeout;

    searchForm.addEventListener("input", function (event) {
        clearTimeout(debounceTimeout);
        console.log("HIII")

        debounceTimeout = setTimeout(() => {
            const search = searchForm.querySelector("[name='search']").value;
            const gender = searchForm.querySelector("[name='gender']").value;
            const spayed = searchForm.querySelector("[name='spayed']").value;
            const isDesc = searchForm.querySelector("[name='isDesc']").checked;
            const pageNo = searchForm.querySelector("[name='pageNo']").value;
            const pageSize = searchForm.querySelector("[name='pageSize']").value;

            const checkboxes = searchForm.querySelectorAll("[name='order[]']");
            const selectedOptions = Array.from(checkboxes)
                .filter(checkbox => checkbox.checked)
                .map(checkbox => checkbox.value);

            const order = selectedOptions.join(",");

            const queryString = `/cat?search=${search}&gender=${gender}&spayed=${spayed}&isDesc=${isDesc}&order=${order}&pageNo=${pageNo}&pageSize=${pageSize}`;

            window.location.href = queryString;
        }, 1000); // Set the debounce delay in milliseconds
    });


    setFormFieldValue(searchForm, 'search', getQueryParam('search'));
    setFormFieldValue(searchForm, 'gender', getQueryParam('gender'));
    setFormFieldValue(searchForm, 'spayed', getQueryParam('spayed'));
    setFormFieldValue(searchForm, 'isDesc', getQueryParam('isDesc'));
    setFormFieldValue(searchForm, 'pageNo', getQueryParam('pageNo'));
    setFormFieldValue(searchForm, 'pageSize', getQueryParam('pageSize'));

    const orderValues = getQueryParam('order');
    if (orderValues) {
        const orderCheckboxes = searchForm.querySelectorAll("[name='order[]']");
        orderCheckboxes.forEach((checkbox) => {
            if (orderValues.split(',').includes(checkbox.value)) {
                checkbox.checked = true;
            }
        });
    }
});

function editCat(catId) {
    const catCard = document.getElementById(`cat-card-${catId}`);

    if (catCard) {
        populateEditForm(catCard, catId);
        openEditCatModal();
    } else {
        console.error("Cat card not found for catId:", catId);
    }
}
function findTextContent(element, searchText) {
    const childElements = element.querySelectorAll(".cat-details");
    for (let i = 0; i < childElements.length; i++) {
        if (childElements[i].textContent.includes(searchText)) {
            const textNode = childElements[i].nextSibling;
            if (textNode && textNode.nodeType === Node.TEXT_NODE) {
                return textNode.textContent.trim();
            }
        }
    }
    return "";
}
function populateEditForm(catCard, catId) {
    const catName = catCard.querySelector(".cat-title").textContent.trim();
    const catGender = catCard.querySelector(".cat-gender").textContent.trim();
    const catSpayed = catCard.querySelector(".cat-spayed").textContent.trim();
    const catColor = catCard.querySelector(".cat-color").textContent.trim();
    const catDescription = catCard.querySelector(".cat-description").textContent.trim();
    const catLocation = catCard.querySelector(".cat-location").textContent.trim();

    // const catImage = catCard.querySelector(".cat-image").src;
    // const catSound = catCard.querySelector(".cat-audio source").src;

    document.getElementById("edit-name").value = catName;
    document.getElementById("edit-gender").value = catGender;
    document.getElementById("edit-color").value = catColor;
    document.getElementById("edit-description").value = catDescription;
    document.getElementById("edit-location").value = catLocation;
    document.getElementById("edit-spayed").value = catSpayed;
    document.getElementById("edit-cat-id").value = catId;
    // document.getElementById("edit-sound_file").value = catSound;
    // document.getElementById("edit-image_file").value = catImage;
}

const editCatModal = document.getElementById("edit-cat-modal");

function openEditCatModal() {
    editCatModal.style.display = "block";
}

function closeEditCatModal() {
    editCatModal.style.display = "none";
}


window.onclick = function (event) {
    if (event.target === editCatModal) {
        closeEditCatModal();
    }
}

