const addSightingModal = document.getElementById("add-sighting-modal");

function openAddSightingModal() {
    addSightingModal.style.display = "block";
}

function closeAddSightingModal() {
    addSightingModal.style.display = "none";
}

const editSightingModal = document.getElementById("edit-sighting-modal");

function openEditSightingModal() {
    editSightingModal.style.display = "block";
}
function closeEditSightingModal() {
    editSightingModal.style.display = "none";
}

function addForm(e) {
    e.preventDefault();
    const addSightingForm = document.getElementById("add-sighting-form");
    const formData = new FormData(addSightingForm);
    let conf = confirm("Are you sure you want to add this sighting?");
    if (!conf) {
        return;
    }
    fetch("/sighting", {
        method: "POST",
        body: formData,
    })
        .then((response) => {
            if (response.ok) {
                console.log(addSightingForm);
                console.log("Sighting added");
                closeAddSightingModal();
                window.location.reload();
            } else {
                console.log("failed");
            }
        })
        .catch((error) => {
            console.error(error);
        })
}



document.addEventListener("DOMContentLoaded", function () {
    const deleteButton = document.querySelectorAll(".delete-button");
    deleteButton.forEach(function (button) {
        button.addEventListener("click", function (event) {
            event.preventDefault();
            console.log("aaaa");
            let conf = confirm("Are you sure you want to delete this sighting?");
            if (!conf) {
                event.preventDefault();
                return;
            }

            const deleteUrl = button.getAttribute("href");
            fetch(deleteUrl, {
                method: "DELETE",
            })
                .then(function (response) {
                    if (response.ok) {
                        window.location.href = "/sighting";
                        location.reload();
                    } else {
                        console.error("Delete failed");
                    }
                })
                .catch(function (error) {
                    console.error("networkError", error);
                })
        })
    })



    const searchForm = document.getElementById("search-form");
    let debounceTimeOut;

    searchForm.addEventListener("input", function (event) {
        event.preventDefault();
        clearTimeout(debounceTimeOut);

        debounceTimeOut = setTimeout(() => {
            const search = searchForm.querySelector("[name='search']").value;
            const isDesc = searchForm.querySelector("[name='isDesc']").checked;
            const pageNo = searchForm.querySelector("[name='pageNo']").value;
            const pageSize = searchForm.querySelector("[name='pageSize']").value;

            const checkboxes = searchForm.querySelectorAll("[name='order[]']");
            console.log(checkboxes);
            const selectedOptions = Array.from(checkboxes)
                .filter(checkbox => checkbox.checked)
                .map(checkbox => checkbox.value);
            const order = selectedOptions.join(",");
            const queryString = `/sighting?search=${search}&isDesc=${isDesc}&order=${order}&pageNo=${pageNo}&pageSize=${pageSize}`;
            window.location.href = queryString;

        }, 1000);
    });
    setFormFieldValue(searchForm, 'search', getQueryParam('search'));
    setFormFieldValue(searchForm, 'isDesc', getQueryParam('isDesc'));
    setFormFieldValue(searchForm, 'pageNo', getQueryParam('pageNo'));
    setFormFieldValue(searchForm, 'pageSize', getQueryParam('pageSize'));

    const orderValues = getQueryParam('order');
    if (orderValues) {
        const orderCheckboxes = searchForm.querySelectorAll("[name='order[]']");
        orderCheckboxes.forEach((checkbox) => {
            if (orderValues.split(',').includes(checkbox.value)) {
                checkbox.checked = true;
            }
        });
    }
})

function editForm(event) {
    event.preventDefault();
    const sightingId = document.getElementById("edit-sighting-id").value;
    const formData = new FormData();
    formData.append("cat_id", document.getElementById("edit-cat-name-select").value);
    formData.append("sighting_location", document.getElementById("edit-sighting-location").value);
    formData.append("date", document.getElementById("edit-date").value);
    formData.append("time", document.getElementById("edit-time").value);
    formData.append("sighting_description", document.getElementById("edit-sighting-description").value);
    formData.append("image_url", document.getElementById("edit-image_url").files[0]);

    let conf = confirm("Are you sure you want to edit this sighting?");
    if (!conf) {
        return;
    }
    fetch(`/sighting/${sightingId}`, {
        method: "POST",
        body: formData,
    })
        .then((response) => {
            if (response.ok) {
                closeEditSightingModal();
                location.reload();
            } else {
                console.error("error editing sighting: ", response)
            }
        })
        .catch((error) => {
            console.error("error editing sighting: ", error);
        });
};

function editSighting(sightingId) {
    const sightingCard = document.getElementById(`sighting-card-${sightingId}`);

    if (sightingId) {
        openEditSightingModal();
    } else {
        console.error("sighting card not found", sightingId);
    }
}

function populateSightingForm(sightingCard, sightingId) {
    const catId = sightingCard.querySelector(".sighting-cat-id").textContent.trim();
    const sightingLocation = sightingCard.querySelector(".sighting-location").textContent.trim();
    const sightingDate = sightingCard.querySelector(".sighting-date").textContent.trim();
    const sightingTime = sightingCard.querySelector(".sighting-time").textContent.trim();
    const sightingDescription = sightingCard.querySelector(".sighting-description").textContent.trim();
    // console.log(catId, sightingId)
    document.getElementById("edit-cat-name-select").value = catId;
    document.getElementById("edit-sighting-location").value = sightingLocation;
    document.getElementById("edit-date").value = sightingDate;
    document.getElementById("edit-time").value = sightingTime;
    document.getElementById("edit-sighting-description").value = sightingDescription;
    document.getElementById("edit-sighting-id").value = sightingId;
}

function editSighting(sightingId) {
    const sightingCard = document.getElementById(`sighting-card-${sightingId}`);

    if (sightingCard) {
        populateSightingForm(sightingCard, sightingId);
        openEditSightingModal();
    } else {
        console.error("Cat card not found for sightingId:", catId);
    }
}

function getQueryParam(name) {
    const urlParams = new URLSearchParams(window.location.search);
    return urlParams.get(name);
}

function setFormFieldValue(searchForm, fieldName, value) {

    const field = searchForm.querySelector(`[name='${fieldName}']`);
    if (field) {
        if (field.type === 'checkbox') {
            // If it's a checkbox, set the 'checked' property
            field.checked = value === 'true';
        } else {
            field.value = value;
        }
    }
}