<?php

require_once PROJECT_ROOT_PATH . "/src/bases/BaseController.php";
require_once PROJECT_ROOT_PATH . "/src/services/CatSrv.php";
require_once PROJECT_ROOT_PATH . "/src/utils/FileUploader.php";
require_once PROJECT_ROOT_PATH . "/src/services/SightingSrv.php";

class CatController extends BaseController
{
  protected static $instance;

  private function __construct($srv)
  {
    parent::__construct($srv);
  }

  public static function getInstance()
  {
    if (!isset(self::$instance)) {
      self::$instance = new static(
        CatSrv::getInstance()
      );
    }
    return self::$instance;
  }

  public function get($urlParams)
  {
    if (!$urlParams) {
      if (!isset($_GET["pageNo"]) || !$_GET["pageNo"] || !$_GET["pageSize"]) {
        header("Location: /cat?pageNo=1&pageSize=10");
      }
      $cats = $this->srv->getAll([
        "search" => $_GET['search'] ?? null,
        "gender" => $_GET['gender'] ?? null,
        "spayed" => $_GET['spayed'] ?? null,
        "order" => $_GET['order'] ?? null,
        "pageNo" => $_GET['pageNo'] ?? null,
        "pageSize" => $_GET['pageSize'] ?? null,
        "isDesc" => $_GET['isDesc'] ?? null
      ]);
      $responseCats = array_map(function ($cat) {
        return $cat->toResponse();
      }, $cats["result"]);

      $_count = $cats["count"];

      // Include the HTML template
      require_once PROJECT_ROOT_PATH . "/public/view/cats.php";

    } else {
      if (!isset($_GET["pageNo"]) || !$_GET["pageNo"] || !$_GET["pageSize"]) {
        header("Location: /cat/" . $urlParams[0] . "?pageNo=1&pageSize=10");
      }
      $cat_id = $urlParams[0];

      $cat = $this->srv->getCatById($cat_id)->toResponse();

      $sightings = SightingSrv::getInstance()->getByCatId(
        [
          "catId" => $cat_id,
          "pageNo" => $_GET['pageNo'] ?? null,
          "pageSize" => $_GET['pageSize'] ?? null,
        ]
      );

      $responseSightings = array_map(function ($sighting) {
        return $sighting->toResponse();
      }, $sightings["result"]);

      $_count = $sightings["count"];


      require_once PROJECT_ROOT_PATH . "/public/view/cat_specific.php";
    }
  }
  public function post($urlParams)
  {
    if ($urlParams) {
      $name = $_POST["name"];
      $description = $_POST['description'];
      $location = $_POST['location'];
      $gender = $_POST['gender'];
      $color = $_POST['color'];
      $spayed = $_POST['spayed'];
      $uploaded_sound = handleFileUpload('sound_file', 'public/sounds/');
      $uploaded_image = handleFileUpload('image_file', 'public/images/');
      $sound_path = null;
      $image_path = null;

      if ($uploaded_sound) {
        $sound_path = "sounds/" . $uploaded_sound;
      }
      if ($uploaded_image) {
        $image_path = "images/" . $uploaded_image;
      }


      $catData = [
        'name' => $name,
        'description' => $description,
        'location' => $location,
        'gender' => $gender,
        'color' => $color,
        'spayed' => $spayed,
      ];
      if ($sound_path !== null) {
        $catData['sound_path'] = $sound_path;
      }
      if ($image_path !== null) {
        $catData['image_path'] = $image_path;
      }


      $cat = $this->srv->updateCat($urlParams[0], $catData);

      if ($cat) {
        $response = new BaseResponse(true, $cat->toResponse(), "Cat successfully updated", 200);
      } else {
        $response = new BaseResponse(false, null, "Failed to insert cat", 400);
      }

      return $response->toJSON();

    }


    $name = $_POST["name"];
    $description = $_POST['description'];
    $location = $_POST['location'];
    $gender = $_POST['gender'];
    $color = $_POST['color'];
    $spayed = $_POST['spayed'];
    $sound_path = "sounds/" . handleFileUpload('sound_file', 'public/sounds/');
    $image_path = "images/" . handleFileUpload('image_file', 'public/images/');


    $catData = [
      'name' => $name,
      'description' => $description,
      'location' => $location,
      'gender' => $gender,
      'color' => $color,
      'spayed' => $spayed,
      'sound_path' => $sound_path,
      'image_path' => $image_path,
    ];


    $cat = $this->srv->createCat($catData);

    if ($cat) {
      $response = new BaseResponse(true, $cat->toResponse(), "Cat successfully inserted", 200);
    } else {
      $response = new BaseResponse(false, null, "Failed to insert cat", 400);
    }

    return $response->toJSON();
  }

  public function delete($urlParams)
  {
    if ($urlParams) {
      $cat_id = $urlParams[0];

      $cat = $this->srv->deleteCat($cat_id);
      $response = new BaseResponse(true, $cat->toResponse(), "successfully deleted", 200);

      return $response->toJSON();
    }
  }
}