<?php

require_once PROJECT_ROOT_PATH . "/src/bases/BaseController.php";
require_once PROJECT_ROOT_PATH . "/src/services/SightingSrv.php";
require_once PROJECT_ROOT_PATH . "/src/utils/FileUploader.php";
require_once PROJECT_ROOT_PATH . "/src/services/CatSrv.php";

class SightingController extends BaseController
{
    protected static $instance;

    private function __construct($srv)
    {
        parent::__construct($srv);
    }

    public static function getInstance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new static(
                SightingSrv::getInstance()
            );
        }
        return self::$instance;
    }

    public function get($urlParams)
    {
        if (!$urlParams) {
            if (!isset($_GET["pageNo"]) || !$_GET["pageNo"] || !$_GET["pageSize"]) {
                header("Location: /sighting?pageNo=1&pageSize=10");
            }
            $sightings = $this->srv->getAll([
                "search" => $_GET['search'] ?? null,
                "order" => $_GET['order'] ?? null,
                "pageNo" => $_GET['pageNo'] ?? null,
                "pageSize" => $_GET['pageSize'] ?? null,
                "isDesc" => $_GET['isDesc'] ?? null
            ]);
            $responseSightings = array_map(function ($sighting) {
                return $sighting->toResponse();
            }, $sightings["result"]);
            $catSrv = (CatSrv::getInstance());
            $cats = $catSrv->getAll([
                "search" => null,
                "gender" => null,
                "spayed" => null,
                "order" => null,
                "pageNo" => null,
                "pageSize" => null,
                "isDesc" => null
            ]);
            $responseCats = array_map(function ($cat) {
                return $cat->toResponse();
            }, $cats["result"]);

            $_count = $sightings["count"];
            require_once PROJECT_ROOT_PATH . "/public/view/sighting.php";

        } else {
            // TODO GET CATS WITH ALL SIGHTINGS DO JOIN.
            return (new BaseResponse(false, null, "NOT IMPLEMENTED", 404))->toJSON();
        }
    }

    public function post($urlParams)
    {
        if ($urlParams) {
            $user_id = $_SESSION['user_id'];
            //        echo $_POST['user_id'];
            $cat_id = $_POST['cat_id'];
            $sighting_location = $_POST['sighting_location'];
            $date = $_POST['date'];
            $time = $_POST['time'];
            $sighting_description = $_POST['sighting_description'];
            $image_url = null;
            $uploaded_file = handleFileUpload('image_url', 'public/images/');
//            echo $uploaded_file;
            if ($uploaded_file !== null) {
                $image_url = "images/" . $uploaded_file;
            }
            $sightingData = [
                'user_id' => $user_id,
                'cat_id' => $cat_id,
                'sighting_location' => $sighting_location,
                'date' => $date,
                'time' => $time,
                'sighting_description' => $sighting_description
            ];

            if ($image_url !== null) {
//                echo ("console.log(".$image_url.")");
                $sightingData['image_url'] = $image_url;
            }

            $sighting = $this->srv->updateSighting($urlParams[0], $sightingData);

            if ($sighting) {
                $response = new BaseResponse(true, $sighting->toResponse(), "Sighting successfully updated", 200);
            } else {
                $response = new BaseResponse(true, $sighting->toResponse(), "fail", 400);
            }

            return $response->toJSON();
        }
        //        $_POST+=['user_id'=>$_SESSION['user_id']];
//        foreach ($_POST)
        $user_id = $_SESSION['user_id'];
        //        echo $_POST['user_id'];
        $cat_id = $_POST['cat_id'];
        $sighting_location = $_POST['sighting_location'];
        $date = $_POST['date'];
        $time = $_POST['time'];
        $sighting_description = $_POST['sighting_description'];
        $image_url = "images/" . handleFileUpload('image_url', 'public/images/');
        $sightingData = [
            'user_id' => $user_id,
            'cat_id' => $cat_id,
            'sighting_location' => $sighting_location,
            'date' => $date,
            'time' => $time,
            'sighting_description' => $sighting_description,
            'image_url' => $image_url,
        ];
//        foreach ($sightingData as $key => $value) {
//            echo ($key . ":" . $value . "<br>");
//        }
        $sighting = $this->srv->createSighting($sightingData);
        if ($sighting) {
            $response = new BaseResponse(true, $sighting->toResponse(), "Sighting successfully updated", 200);
        } else {
            $response = new BaseResponse(false, null, "Failed to insert sighting", 400);
        }
        return $response->toJSON();
    }

    public function delete($urlParams)
    {
        if ($urlParams) {
            $sighting_id = $urlParams[0];

            $sighting = $this->srv->deleteSighting($sighting_id);
            $response = new BaseResponse(true, $sighting->toResponse(), "successfully deleted", 200);
            return $response->toJSON();
        }
    }
}