<?php

require_once PROJECT_ROOT_PATH . "/src/bases/BaseController.php";
require_once PROJECT_ROOT_PATH . "/src/services/UserSrv.php";

class UserController extends BaseController
{
    protected static $instance;

    private function __construct($srv)
    {
        parent::__construct($srv);
    }

    public static function getInstance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new static(
                UserSrv::getInstance()
            );
        }
        return self::$instance;
    }

    public function get($urlParams)
    {
        if (!$urlParams) {
            if (!isset($_GET["pageNo"]) || !isset($_GET["pageSize"])) {
                header("Location: /user?pageNo=1&pageSize=10");
            }
            $users = $this->srv->getAll([
                "order" => "registration_date",
                "pageNo" => $_GET['pageNo'] ?? 1,
                "pageSize" => $_GET['pageSize'] ?? 10
            ]);

            $responseUsers = array_map(function ($user) {
                return $user->toResponse();
            }, $users["result"]);

            $_count = $users["count"];

            // Include the HTML template
            require_once PROJECT_ROOT_PATH . "/public/view/users.php";

        } else {
            // TODO: Implement getting a specific user's data by user ID
            return (new BaseResponse(false, null, "NOT IMPLEMENTED", 404))->toJSON();
        }
    }

    public function post($urlParams)
    {
        if ($urlParams) {
            $userIdFromUrl = $urlParams[0];
            $authenticatedUserId = isset($_SESSION['user_id']) ? $_SESSION['user_id'] : null;

            // Check if the authenticated user is authorized to edit the user's data
            if (!$_SESSION['isAdmin'] && ($authenticatedUserId !== $userIdFromUrl)) {
                return (new BaseResponse(false, null, "You are not authorized to edit this user's data", 403))->toJSON();
            }
            $name = $_POST["nama"];
            $username = $_POST['username'];
            $isAdmin = $_POST['isAdmin'];

            $updatedUserData = [
                "nama" => $name,
                "username" => $username,
                "isAdmin" => $isAdmin
            ];

            $user = $this->srv->updateUser($userIdFromUrl, $updatedUserData);

            if ($user) {
                $response = new BaseResponse(true, $user->toResponse(), "User data updated successfully", 200);
            } else {
                $response = new BaseResponse(false, null, "Failed to update user data", 400);
            }

            return $response->toJSON();
        }
        return (new BaseResponse(false, null, "NOT IMPLEMENTED", 404))->toJSON();
    }

    public function delete($urlParams)
    {
        if ($urlParams) {
            $userIdToDelete = $urlParams[0];
            $authenticatedUserId = isset($_SESSION['user_id']) ? $_SESSION['user_id'] : null;

            if (!$_SESSION['isAdmin'] && $authenticatedUserId !== $userIdToDelete) {
                return (new BaseResponse(false, null, "You are not authorized to delete this user", 403))->toJSON();
            }

            $user = $this->srv->deleteUser($userIdToDelete);

            if ($user) {
                $response = new BaseResponse(true, $user->toResponse(), "User deleted successfully", 200);
            } else {
                $response = new BaseResponse(false, null, "Failed to delete user", 400);
            }

            return $response->toJSON();
        }
        return (new BaseResponse(false, null, "NOT IMPLEMENTED", 404))->toJSON();
    }
}