<?php

require_once PROJECT_ROOT_PATH . "/src/bases/BaseRepository.php";

class SightingRepository extends BaseRepository
{

    protected static $instance;
    protected $tableName = 'sightings';

    private function __construct()
    {
        parent::__construct();
    }

    public static function getInstance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new static();
        }
        return self::$instance;
    }

    public function createSighting($sightingData)
    {
        //        foreach ($sightingData as $key => $value){
//            echo ($key.":" .$value."<br>");
//        }
        return $this->insert(
            $sightingData,
            array(
                'cat_id' => PDO::PARAM_INT,
                'user_id' => PDO::PARAM_INT,
                'sighting_location' => PDO::PARAM_STR,
                'date' => PDO::PARAM_STR,
                'time' => PDO::PARAM_STR,
                'sighting_description' => PDO::PARAM_STR,
                'image_url' => PDO::PARAM_STR
            )
        );
    }

    public function updateSighting($sightingId, $sightingData)
    {
        return $this->update(
            "sighting_id",
            $sightingId,
            $sightingData,
            array(
                'cat_id' => PDO::PARAM_INT,
                'user_id' => PDO::PARAM_INT,
                'sighting_location' => PDO::PARAM_STR,
                'date' => PDO::PARAM_STR,
                'time' => PDO::PARAM_STR,
                'sighting_description' => PDO::PARAM_STR,
                'image_url' => PDO::PARAM_STR
            )
        );
    }

    public function deleteSighting($sighting)
    {
        return $this->delete("sighting_id", $sighting);
    }

    //    public function join(){
//        $sql = "SELECT * FROM sightings s join (cats c, users u) on (c.cat_id=s.cat_id and u.user_id=s.user_id)";
//        $stmt = $this->pdo->prepare($sql);
//        $stmt->execute();
//        return $stmt->fetchAll(PDO::FETCH_ASSOC);
//    }
    public function join($columns = "*", $where = [], $orderBy = [], $page = 1, $perPage = 10, $isDesc = false)
    {
        $query = "SELECT $columns FROM sightings s join (cats c, users u) on (c.cat_id=s.cat_id and u.user_id=s.user_id)";

        if (!empty($where)) {
            $query .= " WHERE " . implode(" AND ", $where);
        }
        if (!empty($orderBy)) {
            $orderByWithDesc = array_map(function ($column) use ($isDesc) {
                return $column . ($isDesc && $isDesc != "false" ? " DESC" : "");
            }, $orderBy);
            $query .= " ORDER BY " . implode(", ", $orderByWithDesc);
        }

        if (!empty($perPage)) {
            $offset = ($page - 1) * $perPage;
            $query .= " LIMIT $perPage OFFSET $offset";
        }

        $stmt = $this->pdo->prepare($query);
        $stmt->execute();
        $result = $stmt->fetchAll(PDO::FETCH_ASSOC);

        $countQuery = "SELECT COUNT(*) as total FROM sightings s join (cats c, users u) on (c.cat_id=s.cat_id and u.user_id=s.user_id)";
        if (!empty($where)) {
            $countQuery .= " WHERE " . implode(" AND ", $where);
        }
        $stmtCount = $this->pdo->prepare($countQuery);
        $stmtCount->execute();
        $totalCount = $stmtCount->fetch(PDO::FETCH_ASSOC)['total'];

        return [
            'count' => $totalCount,
            'result' => $result,
        ];

    }

    public function getSightingById($sightingId)
    {
        return $this->selectOne("*", ["sighting_id = $sightingId"]);
    }
}