<?php

require_once PROJECT_ROOT_PATH . "/src/bases/BaseSrv.php";
require_once PROJECT_ROOT_PATH . "/src/repositories/CatRepository.php";
require_once PROJECT_ROOT_PATH . "/src/models/CatModel.php";

class CatSrv extends BaseSrv
{
    protected static $instance;

    private function __construct($repository)
    {
        parent::__construct();
        $this->repository = $repository;
    }

    public static function getInstance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new static(
                CatRepository::getInstance()
            );
        }
        return self::$instance;
    }

    public function countCats()
    {
        return $this->repository->count();
    }

    public function getAll($config)
    {
        $search = $config['search'];
        $where = [];
        if ($search) {
            $where[] = "(name LIKE '%$search%' OR location LIKE '%$search%')";
        }
        if ($config['gender']) {
            $where[] = "gender = '{$config['gender']}'";
        }
        if ($config['spayed']) {
            $where[] = "spayed = '{$config['spayed']}'";
        }
        $order = [];
        if ($config["order"]) {
            $order = explode(",", $config["order"]);
        }

        $catsResult = $this->repository->select('*', $where, $order, $config["pageNo"], $config['pageSize'], $config['isDesc']);
        $cats = [];
        foreach ($catsResult["result"] as $catResult) {
            $cat = new CatModel();
            $cats[] = $cat->constructFromArray($catResult);
        }
        return ["result" => $cats, "count" => $catsResult["count"]];
    }

    public function createCat($catData)
    {
        $cat = (new CatModel())->constructFromArray($catData);
        $id = $this->repository->createCat($cat->toArray());
        $sqlRes = $this->repository->getCatById($id);
        $cat = new CatModel();

        return $cat->constructFromArray($sqlRes);
    }

    public function getCatById($catId)
    {
        $catData = $this->repository->getCatById($catId);
        if (!$catData) {
            throw new Exception("Cat not found");
        }

        $cat = new CatModel();
        return $cat->constructFromArray($catData);
    }

    public function updateCat($catId, $catData)
    {
        $existingCat = $this->getCatById($catId);
        $updatedCat = (new CatModel())->constructFromArray($catData);

        // Merge updated data into the existing cat model
        foreach (get_object_vars($updatedCat) as $property => $value) {
            if ($value !== null) {
                $existingCat->$property = $value;
            }
        }

        $this->repository->updateCat($catId, $existingCat->toArray());

        return $existingCat;
    }

    public function deleteCat($catId)
    {
        $cat = $this->getCatById($catId);
        $this->repository->delete("cat_id", $catId);
        return $cat;
    }
}