<?php

require_once PROJECT_ROOT_PATH . "/src/bases/BaseSrv.php";
require_once PROJECT_ROOT_PATH . "/src/repositories/SightingRepository.php";
require_once PROJECT_ROOT_PATH . "/src/models/SightingModel.php";

class SightingSrv extends BaseSrv
{
    protected static $instance;
    protected function __construct($repository)
    {
        parent::__construct();
        $this->repository = $repository;
    }

    public static function getInstance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new static(
                SightingRepository::getInstance()
            );
        }
        return self::$instance;
    }
    public function getByCatId($config)
    {
        $where = [];
        $catId = $config["catId"];
        if ($catId) {
            $where[] = "(s.cat_id = '$catId')";
        }

        $order = ["date", "time"];

        $res = $this->repository->join("*", $where, $order, $config["pageNo"], $config['pageSize'], true);

        $sightings = [];

        foreach ($res['result'] as $re) {
            $sighting = new SightingModel();
            $sightings[] = $sighting->constructFromArray($re);
        }
        return ["result" => $sightings, "count" => $res["count"]];
    }

    public function getAll($config)
    {
        $search = $config['search'];
        $where = [];
        if ($search) {
            $where[] = "(name LIKE '%$search%' OR sighting_location LIKE '%$search%' OR username LIKE '%$search%')";
        }

        $order = [];
        if ($config["order"]) {
            $order = explode(",", $config["order"]);
        }

        $res = $this->repository->join("*", $where, $order, $config["pageNo"], $config['pageSize'], $config['isDesc']);


        $sightings = [];

        foreach ($res['result'] as $re) {
            $sighting = new SightingModel();
            $sightings[] = $sighting->constructFromArray($re);
        }
        return ["result" => $sightings, "count" => $res["count"]];
    }

    public function createSighting($sightingData)
    {
        $sighting = (new SightingModel())->constructFromArray($sightingData);
        $id = $this->repository->createSighting($sighting->toArray());
        $sqlRes = $this->repository->getSightingById($id);
        $sighting = new SightingModel();
        return $sighting->constructFromArray($sqlRes);
    }

    public function deleteSighting($sightingId)
    {
        $sighting = $this->getSightingById($sightingId);
        $this->repository->delete("sighting_id", $sightingId);
        return $sighting;
    }

    public function getSightingById($sightingId)
    {
        $sightingData = $this->repository->getSightingById($sightingId);
        if (!$sightingData) {
            throw new Exception("sighting not found");
        }

        $sighting = new SightingModel();
        return $sighting->constructFromArray($sightingData);
    }

    public function updateSighting($sightingId, $sightingData)
    {
        $existingSighting = $this->getSightingById($sightingId);
        $updatedSighting = (new SightingModel())->constructFromArray($sightingData);

        foreach (get_object_vars($updatedSighting) as $property => $value) {
            if ($value !== null) {
                $existingSighting->$property = $value;
            }
        }

        $this->repository->updateSighting($sightingId, $existingSighting->toArray());

        return $existingSighting;
    }
}