<?php

require_once PROJECT_ROOT_PATH . "/src/bases/BaseSrv.php";
require_once PROJECT_ROOT_PATH . "/src/repositories/UserRepository.php";
require_once PROJECT_ROOT_PATH . "/src/models/UserModel.php";

class UserSrv extends BaseSrv
{
  protected static $instance;

  protected function __construct($repository)
  {
    parent::__construct();
    $this->repository = $repository;
  }

  public static function getInstance()
  {
    if (!isset(self::$instance)) {
      self::$instance = new static(
        UserRepository::getInstance()
      );
    }
    return self::$instance;
  }

  public function getAllUser()
  {
    $usersSQL = $this->repository->findAll();
    $users = [];
    foreach ($usersSQL as $userSQL) {
      $user = new UserModel();
      $users[] = $user->constructFromArray($userSQL);
    }
    return $users;
  }

  public function register($nama, $username, $email, $password, $confirm_password)
  {
    $existingUser = $this->getByEmail($email);
    if (!is_null($existingUser->get('email'))) {
      throw new Exception("Email already taken!");
    }
    if ($password !== $confirm_password) {
      throw new Exception("Password and confirm password do not match");
    }
    //    echo ("<script>console.log(".$username.")</script>");
    $u = $this->getByUsername($username);
    //    echo is_null($u);
    if (!is_null($u->get('username'))) {
      //        echo $username;
      throw new Exception("Username already taken!");
    }

    $user = (new UserModel())->set('nama', $nama)->set('username', $username)->set('email', $email)->set('password', password_hash($password, PASSWORD_DEFAULT));

    $id = $this->repository->insert(
      $user->toArray(),
      array(
        'nama' => PDO::PARAM_STR,
        'username' => PDO::PARAM_STR,
        'email' => PDO::PARAM_STR,
        'password' => PDO::PARAM_STR,
      )
    );
    $sqlRes = $this->repository->getById($id);
    $user = new UserModel();

    return $user->constructFromArray($sqlRes);
  }

  public function getAll($config)
  {
    $order = ["registration_date"];

    $usersResult = $this->repository->select('*', [], $order, $config["pageNo"], $config['pageSize'], null);
    $users = [];
    foreach ($usersResult["result"] as $userResult) {
      $user = new UserModel();
      $users[] = $user->constructFromArray($userResult);
    }
    return ["result" => $users, "count" => $usersResult["count"]];
  }


  public function login($email_uname, $password)
  {
    $user = $this->getByEmail($email_uname);
    if (is_null($user)) {
      throw new Exception("User not found");
    }

    if (!password_verify($password, $user->get('password') ?? "")) {
      throw new Exception("Password do not match");
    }

    $_SESSION["user_id"] = $user->get('user_id');
    $_SESSION["isAdmin"] = $user->get('isAdmin');
    $_SESSION['username'] = $user->get('username');


    return $user;
  }
  public function getUserById($userId)
  {
    $userData = $this->repository->getUserById($userId);
    if (!$userData) {
      throw new Exception("User not found");
    }

    $user = new UserModel();
    return $user->constructFromArray($userData);
  }

  public function deleteUser($userId)
  {
    $cat = $this->getUserById($userId);
    $this->repository->delete("user_id", $userId);
    return $cat;
  }

  public function getByEmail($email)
  {
    $user = new UserModel();
    $sqlRes = $this->repository->getByEmail($email);
    if ($sqlRes) {
      $user->constructFromArray($sqlRes);
    }

    return $user;
  }

  public function getByUsername($username)
  {
    $user = new UserModel();
    $sqlRes = $this->repository->getByUsername($username);
    if ($sqlRes) {
      $user->constructFromArray($sqlRes);
    }
    return $user;
  }

  public function updateUser($userId, $userData)
  {
    $existingUser = $this->getUserById($userId);
    $updatedUser = (new UserModel())->constructFromArray($userData);

    // Merge updated data into the existing cat model
    foreach (get_object_vars($updatedUser) as $property => $value) {
      if ($value !== null) {
        $existingUser->$property = $value;
      }
    }

    $this->repository->updateUser($userId, $existingUser->toArray());

    return $existingUser;
  }
}